package com.me.common.ecs;

import com.badlogic.gdx.utils.Array;

public class Engine {

    private Array<Entity> entities;
    private ComponentBag[] components;
    private Array<EntitySystem> systems;

    public Engine() {
        this.entities = new Array<>();
        this.systems = new Array<>();
    }

    public void registerComponentClass(Class<? extends Component> clazz) {
        ComponentType.registerComponentType(clazz);
    }

    public void registerSystem(EntitySystem system) {
        this.systems.add(system);
    }

    public void ready() {
        this.components = new ComponentBag[ComponentType.getRegisteredComponentTypeCount()];
        for (int i = 0; i < components.length; i++) {
            components[i] = new ComponentBag();
        }
    }

    public void update(float dt) {
        for (EntitySystem system : systems) {
            system.preProcessing();
            updateSystem(system, dt);
            system.postProcessing();
        }
    }

    public Entity createEntity() {
        Entity entity = new Entity(this);
        entities.add(entity);
        return entity;
    }

    public void removeEntity(Entity entity) {
        removeAllEntityComponents(entity.getId());
        entities.removeValue(entity, true);
    }

    private void removeAllEntityComponents(int entityId) {
        for (int i = 0; i < components.length; i++) {
            components[i].insert(entityId, null);
        }
    }

    protected void addEntityComponent(Entity entity, Component component) {
        ComponentType type = ComponentType.getComponentType(component.getClass());
        components[type.getId()].insert(entity.getId(), component);
        entity.addComponentType(type);
    }

    protected void removeEntityComponent(Entity entity, Component component) {
        ComponentType type = ComponentType.getComponentType(component.getClass());
        components[type.getId()].remove(entity.getId());
        entity.removeComponentType(type);
    }

    protected <T extends Component> T getEntityComponent(Entity entity, Class<T> clazz) {
        ComponentType type = ComponentType.getComponentType(clazz);
        return clazz.cast(components[type.getId()].get(entity.getId()));
    }

    protected void updateSystem(EntitySystem system, float dt) {
        for (Entity entity : entities) {
            if (!entity.isActive()) {
                continue;
            }

            // Check if this system is interested in this entity
            if ((entity.getComponentBits() & system.getTypeMask()) != system.getTypeMask()) {
                continue;
            }

            // If so, process the entity
            system.processEntity(entity, dt);
        }
    }


}
